{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Import libraries"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import ee\n",
    "import geemap"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Create an interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map = geemap.Map(center=[40, -100], zoom=4)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Add Earth Engine Python script"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Add Earth Engine dataset\n",
    "image = ee.Image(\"USGS/SRTMGL1_003\")\n",
    "\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "#  Chapter:      A1.3 Built Environments\n",
    "#  Checkpoint:   A13b\n",
    "#  Author:       Erin Trochim\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "\n",
    "# Import roads data.\n",
    "grip4_africa = ee.FeatureCollection(\n",
    "        'projects/sat-io/open-datasets/GRIP4/Africa'),\n",
    "    grip4_north_america = ee.FeatureCollection(\n",
    "        'projects/sat-io/open-datasets/GRIP4/North-America'),\n",
    "    grip4_europe = ee.FeatureCollection(\n",
    "        'projects/sat-io/open-datasets/GRIP4/Europe')\n",
    "\n",
    "# Check the roads data sizes.\n",
    "print('Grip4 Africa size', grip4_africa.size())\n",
    "print('Grip4 North America size', grip4_north_america.size())\n",
    "print('Grip4 Europe size', grip4_europe.size())\n",
    "\n",
    "# Display the roads data.\n",
    "Map.addLayer(ee.FeatureCollection(grip4_africa).style(**{\n",
    "    'color': '413B3A',\n",
    "    'width': 1\n",
    "}), {}, 'Grip4 Africa')\n",
    "Map.addLayer(ee.FeatureCollection(grip4_north_america).style(**{\n",
    "    'color': '413B3A',\n",
    "    'width': 1\n",
    "}), {}, 'Grip4 North America')\n",
    "Map.addLayer(ee.FeatureCollection(grip4_europe).style(**{\n",
    "    'color': '413B3A',\n",
    "    'width': 1\n",
    "}), {}, 'Grip4 Europe')\n",
    "\n",
    "# Import simplified countries.\n",
    "countries = ee.FeatureCollection('USDOS/LSIB_SIMPLE/2017')\n",
    "\n",
    "# Add a function to calculate the feature's geometry area.\n",
    "# Add the function as a property.\n",
    "def addArea(feature):\n",
    "    return feature.set({\n",
    "        'areaKm': feature.geometry().area().divide(1000 *\n",
    "            1000)\n",
    "    }); # km2 squared\n",
    "\n",
    "\n",
    "# Map the area getting function over the FeatureCollection.\n",
    "countriesArea = countries.map(addArea)\n",
    "\n",
    "# Filter to the largest country in Africa.\n",
    "Algeria = countriesArea.filter(ee.Filter.inList('country_na', [\n",
    "    'Algeria'\n",
    "]))\n",
    "\n",
    "# Display the selected countries.\n",
    "Map.addLayer(Algeria.style(**{\n",
    "    'fillColor': 'b5ffb4',\n",
    "    'color': '00909F',\n",
    "    'width': 1.0\n",
    "}), {}, 'Algeria')\n",
    "\n",
    "# This function calculates the road length per country for the associated GRIP dataset.\n",
    "def roadLength4Country(country, grip4):\n",
    "\n",
    "    # Join roads to countries.\n",
    "    intersectsFilter = ee.Filter.intersects({\n",
    "        'leftField': '.geo',\n",
    "        'rightField': '.geo',\n",
    "        'maxError': 10\n",
    "    })\n",
    "\n",
    "    grip4Selected = grip4.filterBounds(country)\n",
    "\n",
    "    countriesWithRds = ee.Join.saveAll('roads').apply({\n",
    "        'primary': country,\n",
    "        'secondary': grip4Selected,\n",
    "        'condition': intersectsFilter\n",
    "    }).filter(ee.Filter.neq('roads', None))\n",
    "\n",
    "    # Return country with calculation of roadLength and roadsPerArea.\n",
    "\n",
    "def func_gfe(country):\n",
    "        roadsList = ee.List(country.get('roads'))\n",
    "        roadLengths = roadsList.map(function(road) {\n",
    "            return ee.Feature(road).intersection(\n",
    "                country, 10).length(10)\n",
    "        })\n",
    "        roadLength = ee.Number(roadLengths.reduce(ee \\\n",
    "            .Reducer.sum()))\n",
    "        return country.set({\n",
    "            'roadLength': roadLength.divide(\n",
    "                1000), # Convert to km.\n",
    "            'roadsPerArea': roadLength.divide(ee \\\n",
    "                .Number(country.get('areaKm'))\n",
    "            )\n",
    "        })\n",
    "\n",
    "    return countriesWithRds.map(func_gfe\n",
    ").select(['country_na', 'areaKm', 'roadLength',\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    ").select(['country_na', 'areaKm', 'roadLength',\n",
    "        'roadsPerArea'\n",
    "    ])\n",
    "\n",
    "\n",
    "# Apply the road length function to Algeria.\n",
    "roadLengthAlgeria = roadLength4Country(Algeria, grip4_africa)\n",
    "\n",
    "# Print the road statistics for Algeria.\n",
    "print('Roads statistics in Algeria', roadLengthAlgeria)\n",
    "\n",
    "#  -----------------------------------------------------------------------\n",
    "#  CHECKPOINT\n",
    "#  -----------------------------------------------------------------------\n",
    "\n",
    "# Export feature collection to drive.\n",
    "Export.table.toDrive({\n",
    "    'collection': roadLengthAlgeria,\n",
    "    'description': 'RoadStatisticsforAlgeria',\n",
    "    'selectors': ['country_na', 'roadLength', 'roadsPerArea']\n",
    "})\n",
    "\n",
    "# Print the first feature of the grip4 Africa feature collection.\n",
    "print(grip4_africa.limit(1))\n",
    "\n",
    "Map.setCenter(-0.759, 9.235, 6)\n",
    "Map.addLayer(grip4_africa.limit(1),\n",
    "    {},\n",
    "    'Road length comparison')\n",
    "\n",
    "# This function adds line length in km.\n",
    "def addLength(feature):\n",
    "    return feature.set({\n",
    "        'lengthKm': feature.length().divide(1000)\n",
    "    })\n",
    "\n",
    "\n",
    "# Calculate the line lengths for all roads in Africa.\n",
    "grip4_africaLength = grip4_africa.map(addLength)\n",
    "\n",
    "# Compare with other values.\n",
    "print('Calculated road length property', grip4_africaLength.limit(1))\n",
    "\n",
    "# Repeat the analysis to calculate the length of all roads.\n",
    "# Filter the table geographically: only keep roads in Algeria.\n",
    "grip4_Algeria = grip4_africaLength.filterBounds(Algeria)\n",
    "\n",
    "# Visualize the output.\n",
    "Map.addLayer(grip4_Algeria.style(**{\n",
    "    'color': 'green',\n",
    "    'width': 2.0\n",
    "}), {}, 'Algeria roads')\n",
    "\n",
    "# Sum the lengths for roads in Algeria.\n",
    "sumLengthKmAlgeria = ee.Number(\n",
    "    # Reduce to get the sum.\n",
    "    grip4_Algeria.reduceColumns(ee.Reducer.sum(), ['lengthKm']) \\\n",
    "    .get('sum')\n",
    ")\n",
    "\n",
    "# Print the result.\n",
    "print('Length of all roads in Algeria', sumLengthKmAlgeria)\n",
    "\n",
    "#  -----------------------------------------------------------------------\n",
    "#  CHECKPOINT\n",
    "#  -----------------------------------------------------------------------"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Display the interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map"
   ]
  }
 ],
 "metadata": {
  "anaconda-cloud": {},
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
